/*
 * Implementation of class ImageArray
 *
 * $Date: 2004/05/21 09:22:58 $
 * $Revision: 1.8 $
 *
 */

#ifndef IMAGE_ARRAY_CC__
#define IMAGE_ARRAY_CC__

#include "WImage/debug.h"
#include "WImage/miscdefs.h"
#include "WImage/ImageArray.hh"
#include <stdexcept>
#include <iostream>

template <class T>
ImageArray<T>::ImageArray (int rows, int cols) 
{
  m_ar = new T[rows*cols];
  m_rows = NULL;
  m_ysize = rows;
  m_xsize = cols;
  m_xysize = rows*cols;
  for (int i = 0; i < m_xysize; i++)
    {
      m_ar[i] = (T)0;
    }
  updateRowsArray ();
}

template <class T>
ImageArray<T>::~ImageArray (void) 
{
  delete [] m_ar;
  delete m_rows;
}

template <class T>void 
ImageArray<T>::resize (int rows, int cols)
{
  int size = rows*cols;
  int ymin = MIN (rows, m_ysize);
  int xmin = MIN (cols, m_xsize);
  int x = 0, y = 0;
  if (rows < 0 || cols < 0)
    {
      throw std::invalid_argument ("resize: rows and cols cannot "
                                   "have negative values.");
    }
  if (rows + cols == 0)
    {
      throw std::invalid_argument ("resize: "
                                   "array size cannot be zero.");
    }

  T *tmp = new T[size];

  m_ysize = rows;
  m_xsize = cols;
  m_xysize = size;

  int *oldRows = m_rows;
  m_rows = NULL;
  updateRowsArray ();

  //DPRINTF (("resize (%d, %d)\n", m_ysize, m_xsize));
  
  for (y = 0; y < ymin; y++)
    {
      int y1offs = m_rows[y];
      int y2offs = oldRows[y];
      for (x = 0; x < xmin; x++)
        {
          //DPRINTF (("tmp[%d+%d] = m_ar[%d+%d]: %d\n", 
          //          y1offs, x, y2offs, x, (int)m_ar[y2offs]));
          tmp[y1offs + x] = m_ar[y2offs + x];
        }
      for (; x < m_xsize; x++)
        {
          //DPRINTF (("%d < %d; tmp[%d+%d] = (T)0\n", x, m_xsize, y1offs, x));
          tmp[y1offs + x] = (T)0;
        }
    }
  
  for (; y < m_ysize; y++)
    {
      int y1offs = m_rows[y];
      for (x = 0; x < m_xsize; x++)
        {
          //DPRINTF (("%d < %d; tmp[%d+%d] = (T)0\n", y, rows, y1offs, x));
          tmp[y1offs + x] = (T)0;
        }
    }

  delete [] m_ar;
  m_ar = tmp;
  delete [] oldRows;
}

template <class T>void 
ImageArray<T>::import (int rows, int cols, T *array)
{
  delete [] m_ar;
  m_ar = array;
  m_ysize = rows;
  m_xsize = cols;
  m_xysize = cols*rows;
  updateRowsArray ();
}

template <class T>ImageArray<T> *
ImageArray<T>::clone (void) const
{
  ImageArray<T> *clone = new ImageArray (m_ysize, m_xsize);
  for (int row = 0; row < m_ysize; row++)
    {
      for (int col = 0; col < m_xsize; col++)
        {
          clone->to (row, col, m_ar[m_rows[row] + col]);
        }
    }
  return clone;
}

template <class T>void
ImageArray<T>::copy (ImageArray<T> &ia) 
{
  resize (ia.rows (), ia.cols ());
  for (int i = 0; i < ia.size (); i++)
    {
      to (i, ia.at (i));
    }
}

template <class T>bool
ImageArray<T>::equals (ImageArray<T> &ia) const
{
  return this->epsilons (ia, (T)0);
}

template <class T>bool
ImageArray<T>::epsilons (ImageArray<T> &ia, T epsilon) const
{
  if (!(ia.rows () == this->rows () 
        && ia.cols () == this->cols ()
        && ia.size () == this->size ()))
    {
      return false;
    }
  for (int row = 0; row < ia.rows (); row++)
    {
      for (int col = 0; col < ia.cols (); col++)
        {
          if (ia.at (row, col) > this->at (row, col))
            {
              if (ia.at (row, col) - this->at (row, col) > epsilon)
                {
                  return false;
                }
            }
          else
            {
              if (this->at (row, col) - ia.at (row, col) > epsilon)
                {
                  return false;
                }
            }
        }
    }
  return true;
}

template <class T>T
ImageArray<T>::smax (void) const
{
  T max;
  T cur;
  
  max = cur = m_ar[0];
  
  for (int i = 0; i < this->m_xysize; i++, cur = m_ar[i])
    {
      if (cur > max)
        {
          max = cur;
        }
    }
  return max;
}

template <class T>T
ImageArray<T>::smin (void) const
{
  T min;
  T cur;
  
  min = cur = m_ar[0];
  
  for (int i = 0; i < this->m_xysize; i++, cur = m_ar[i])
    {
      if (cur < min)
        {
          min = cur;
        }
    }
  return min;
}

template <class T>T
ImageArray<T>::amax (void) const
{
  T max;
  T cur;
  
  max = cur = (m_ar[0] < 0? -1*m_ar[0]: m_ar[0]);

  for (int i = 0; i < this->m_xysize; 
       i++, cur = (m_ar[i] < 0? -1 * m_ar[i]: m_ar[i]))
    {
      if (cur > max)
        {
          max = cur;
        }
    }
  return max;
}

template <class T>T
ImageArray<T>::amin (void) const
{
  T min;
  T cur;
  
  min = cur = (m_ar[0] < 0? -1 * m_ar[0]: m_ar[0]);

  for (int i = 0; i < this->m_xysize; 
       i++, cur = (m_ar[i] < 0? -1 * m_ar[i]: m_ar[i]))
    {
      if (cur < min)
        {
          min = cur;
        }
    }
  return min;
}

template <class T>T
ImageArray<T>::saverage (int fromY, int fromX, int toY, int toX) const
{
  T ret = 0;
  int nVals = 0;
  int lower = (toY < 0)? m_ysize: toY;
  int right = (toX < 0)? m_xsize: toX;

  for (int y = (fromY < 0? 0: fromY); y < lower && y < m_ysize; y++)
    {
      for (int x = (fromX < 0? 0: fromX); x < right && x < m_xsize; x++)
        {
          ++nVals;
          ret += at (y, x);
        }
    }
  
  return (nVals > 0)? ret / static_cast < T > (nVals): 0;
}

template <class T>T
ImageArray<T>::aaverage (int fromY, int fromX, int toY, int toX) const
{
  T ret = 0;
  int nVals = 0;
  int lower = (toY < 0)? m_ysize: toY;
  int right = (toX < 0)? m_xsize: toX;

  for (int y = (fromY < 0? 0: fromY); y < lower && y < m_ysize; y++)
    {
      for (int x = (fromX < 0? 0: fromX); x < right && x < m_xsize; x++)
        {
          ++nVals;
          T tmp = at (y, x);
          ret += tmp < 0? -tmp: tmp;
        }
    }
  
  return (nVals > 0)? ret / static_cast < T > (nVals): 0;
}


template <class T>void 
ImageArray<T>::updateRowsArray (void)
{
  if (m_rows != NULL)
    {
      delete m_rows;
    }
  m_rows = new int [m_ysize];
  for (int i = 0; i < m_ysize; i++)
    {
      m_rows[i] = m_xsize * i;
    }
}

#endif /* IMAGE_ARRAY_CC__ */
