/*
 * Implementation of class JpgWriter
 *
 * $Date: 2003/09/16 16:46:45 $
 * $Revision: 1.5 $
 *
 */

#include "WImage/JpgWriter.hh"
#include "WImage/debug.h"
#include "WImage/tools.h"
#include <stdio.h>
extern "C" {
#include <jpeglib.h>
}

static int writeJpegFile (FILE *outFile, unsigned char *allpixels, int rows, 
                          int cols, clrmodel cmodel, int chan, int quality);
int 
JpgWriter::writefmt (void)
{
  int rc = 0;
  FILE *outFile;
  
  if (m_fname == NULL)
    {
      outFile = stdout;
    }
  else 
    {
      /*
       * Never fails since checked before.
       */
      outFile = fopen (m_fname, "wb");
    }

  int xysize = m_images[0]->size ();
  
  pixel *allpixels = new pixel [m_channels * xysize];

  /* import */
  for (int i = 0; i < xysize; i++)
    {
      for (int j = 0; j < m_channels; j++)
        {
          allpixels[m_channels * i + j] 
            = tools_coeff2pixel (m_images[j]->at (i));
        }
    }

  /* write */
  rc = writeJpegFile (outFile, allpixels, m_images[0]->rows (), 
                      m_images[0]->cols (), m_cmodel, m_channels, m_quality);
  fclose (outFile);

  /* cleanup */
  delete [] allpixels;
  return rc;
}

/*
 * The following code is derived from the example.c file shipping with
 * the JPEG-lib distribution. 
 */

static int 
writeJpegFile (FILE *outFile, unsigned char *allpixels, int rows, 
               int cols, clrmodel cmodel, int channels, int quality)
{
  /* This struct contains the JPEG compression parameters and pointers to
   * working space (which is allocated as needed by the JPEG library).
   * It is possible to have several such structures, representing multiple
   * compression/decompression processes, in existence at once.  We refer
   * to any one struct (and its associated working data) as a "JPEG object". */
  struct jpeg_compress_struct cinfo;

  /* This struct represents a JPEG error handler.  It is declared separately
   * because applications often want to supply a specialized error handler
   * (see the second half of this file for an example).  But here we just
   * take the easy way out and use the standard error handler, which will
   * print a message on stderr and call exit() if compression fails.
   * Note that this struct must live as long as the main JPEG parameter
   * struct, to avoid dangling-pointer problems. */
  struct jpeg_error_mgr jerr;

  /* More stuff */
  JSAMPROW rowPointer[1];	/* pointer to JSAMPLE row[s] */
  int rowStride;		/* physical row width in image buffer */
  J_COLOR_SPACE cSpace;

  /* Set the correct color space according to the arguments */
  switch (cmodel)
    {
    case cm_rgb:
      cSpace = JCS_RGB; 	/* colorspace of input image */
      break;
    case cm_yuv:
      cSpace = JCS_YCbCr;       /* colorspace of input image */
      break;
    default:
      return -1;
    }

  /* Step 1: allocate and initialize JPEG compression object */

  /* We have to set up the error handler first, in case the initialization
   * step fails.  (Unlikely, but it could happen if you are out of memory.)
   * This routine fills in the contents of struct jerr, and returns jerr's
   * address which we place into the link field in cinfo. */
  cinfo.err = jpeg_std_error (&jerr);

  /* Now we can initialize the JPEG compression object. */
  jpeg_create_compress (&cinfo);

  /* Step 2: specify data destination (eg, a file) */
  /* Note: steps 2 and 3 can be done in either order. */

  /* Here we use the library-supplied code to send compressed data to a
   * stdio stream.  You can also write your own code to do something else.
   * VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
   * requires it in order to write binary files. */
  jpeg_stdio_dest (&cinfo, outFile);

  /* Step 3: set parameters for compression */

  /* First we supply a description of the input image.
   * Four fields of the cinfo struct must be filled in: */
  cinfo.image_width = cols;        	/* image width and height, in pixels */
  cinfo.image_height = rows;        
  cinfo.input_components = channels;	/* # of color components per pixel */
  cinfo.in_color_space = cSpace;
  
  /* Now use the library's routine to set default compression parameters.
   * (You must set at least cinfo.in_color_space before calling this,
   * since the defaults depend on the source color space.) */
  jpeg_set_defaults (&cinfo);

  /* Now you can set any non-default parameters you wish to.
   * Here we just illustrate the use of quality (quantization table): */
  jpeg_set_quality (&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  /* Step 4: Start compressor */

  /* TRUE ensures that we will write a complete interchange-JPEG file.
   * Pass TRUE unless you are very sure of what you're doing. */
  jpeg_start_compress (&cinfo, TRUE);

  /* 
   * Step 5: while (scan lines remain to be written) 
   *           jpeg_write_scanlines(...); 
   */

  /* Here we use the library's state variable cinfo.next_scanline as the
   * loop counter, so that we don't have to keep track ourselves.
   * To keep things simple, we pass one scanline per call; you can pass
   * more if you wish, though. */
  rowStride = cols * channels;	/* JSAMPLEs per row in allpixels */

  while (cinfo.next_scanline < cinfo.image_height) 
    {
    /* jpeg_write_scanlines expects an array of pointers to scanlines.
     * Here the array is only one element long, but you could pass
     * more than one scanline at a time if that's more convenient. */
      rowPointer[0] = & allpixels[cinfo.next_scanline * rowStride];
      (void) jpeg_write_scanlines (&cinfo, rowPointer, 1);
    }

  /* Step 6: Finish compression */

  jpeg_finish_compress (&cinfo);

  /* Step 7: release JPEG compression object */

  /* This is an important step since it will release a good deal of memory. */
  jpeg_destroy_compress (&cinfo);

  /* And we're done! */
  return 0;
}

