/*
 * class PixmapWriter
 *
 * $Date: 2003/11/14 14:57:40 $
 * $Revision: 1.5 $
 *
 */

#ifndef PIXMAP_WRITER_HH__
#define PIXMAP_WRITER_HH__

#include "WImage/PixmapFile.hh"

/** @addtogroup WImage */

/**
 * @class PixmapWriter
 * An abstract image writer. The image is being written putting its content
 * from an array of greyscale image objects into the file. 
 */
/*abstract*/
class 
PixmapWriter: public PixmapFile
{
public:
  
  /** Constructor. Only calls the mother class' constructor to initialize
   * the {\em ImageArray} reference. 
   * @param name
   * the file name
   * @param images
   * the array of greyscale images for the values 
   * @param channels
   * the number of colors */
  PixmapWriter (char const *name, Image *images[], int channels)
    : PixmapFile (name, images, channels) {}
  /** Write the image. All steps independent of the file format will be 
   * performed, like testing for file writeability etc. 
   * @exception ios_base::failure
   * a read error has occured [not supported by all libraries, so eventually 
   * {\em invalid_argument} instead] */
  void write (void);

protected: 
  /** Write different file formats (abstract). This is the method to be 
   * implemented for every image file format. It will be called by the
   * {\em write ()} method. 
   * @return 
   * 0 if successful, -1 on write error. */
  virtual int writefmt (void) = 0;
  
}; /* class PixmapWriter */

#endif /* PIXMAP_WRITER_HH__ */
