/*
 * class RowVector
 *
 * $Date: 2004/05/20 11:06:12 $
 * $Revision: 1.6 $
 *
 */

#ifndef ROW_VECTOR_HH__
#define ROW_VECTOR_HH__

#include "WImage/ImageVector.hh"

/** @addtogroup WImage */

/**
 * @class RowVector
 * Row-reference. A reference to a two-dimensional array's row. 
 */
class 
RowVector: public ImageVector
{
public:
  
  /** Constructor. As we've got no default constructor we need to 
   * call the mother's class' constructor here... */
  RowVector (ImageArray<coeff> *ar) : ImageVector (ar) { go (0); }
  /** Destructor. Does nothing. */
  virtual ~RowVector (void) { } 

  /** Sets a new root. In this context this ``new root'' means a new row.
   * @exception invalid_argument
   * the new root is greater than the number of rows
   * @param root 
   * the new row */
  virtual void go (int root);
  /** Get a value. Return the value at a given column from the current row.
   * @exception invalid_argument
   * a negative value was given for the new root
   * @param pos
   * the column 
   * @return the value */
  virtual coeff at (int pos);
  /** Assigns a new value. Sets a new value at a given column from 
   * the current row.
   * @param pos
   * the column
   * @param val
   * the new value */
  virtual void to (int pos, coeff val);
  /** The vector's size. Returns the vector's size (the number of cols)
   * @return the number of cols */
  virtual int size (void);
  /** Updates the vector's settings. This is necessary each time after
   * the array has been resized. */
  virtual void update (void);

private:
  /** Internal value. The base to compute the row's entries (row*xsize) */
  int m_rbase;
}; /* class RowVector */

#endif /* ROW_VECTOR_HH__ */
