/*
 * class StandardTransform
 *
 * $Date: 2004/05/13 17:51:18 $
 * $Revision: 1.6 $
 *
 */

#ifndef STANDARD_TRANSFORM_HH__
#define STANDARD_TRANSFORM_HH__

#include "Wave/WaveletTransform.hh"

/** @addtogroup Wave */

/** 
 * @class StandardTransform
 * A Standard Transform. The transform is two-dimensional, 
 * it thus works on images.
 */
class 
StandardTransform : public WaveletTransform
{
public:
  /** Constructor. Calls the superclass' constructor. 
   * @param img
   * the image
   * @param fil
   * the filter (wavelet) */
  StandardTransform (Image &img, FilterSet &fil) 
    : WaveletTransform (img, fil) {}

protected:
  /** Perform a Standard transform on the image. 
   * If a symmetric filter is used, an image can't get fully decomposed. In
   * this case the maximum number of steps is automatically limited to 
   * level - 1.
   * @param steps
   * the number of transform steps */
  virtual void doanalysis (int steps);
  /** Perform an inverse Standard transform on the image. 
   * @param steps
   * the number of inverse transform steps 
   * @param prevSteps 
   * if greater than 0, the number of previous decomposition steps
   * to assume. This is necessary to reconstruct images that are
   * not square and have side lengths which are not powers of two. */
  virtual void dosynthesis (int steps, int prevSteps = 0);
}; /* class StandardTransform */

#endif /* STANDARD_TRANSFORM_HH__ */


