/*
 * Compare two PGM images
 *
 * $Date: 2003/10/17 14:33:39 $
 * $Revision: 1.3 $
 *
 */

#include <iostream>
#include <iomanip>
#include <stdexcept>
#include <assert.h>
#include <stdio.h>
#include <math.h>
#include <WTools.hh>

#ifndef __WATCOMC__
using namespace std;
#endif

static bool
power2 (int num)
{
  int i;
  for (i = 2; i < num; i *= 2)
    {
    }
  return i == num;
}

static void
sizecheck (Image *img)
{
  int x = img->cols (), y = img->rows ();
  if (x != y || !power2 (x))
    {
      int newsize = x < y? y: x;
      if (!power2 (newsize))
        {
          int i;
          for (i = 2; i < newsize; i *= 2)
            {
            }
          newsize = i;
        }
      img->resize (newsize, newsize);
    }
}

int
main (int argc, char *argv[])
{
  Image *img1 = new StillImage ();
  Image *img2 = new StillImage ();
  double percent = 1;
  FilterSet &filter = Haar;
  imgtype type = SCANNED;
  
  /* Preliminary steps */
  
  cerr << "pgmcompare version " << WAVE_VERSION << " (c) 2001-2003 by Herbert"
#ifdef PFI
       << " (pfi support enabled)"
#endif
       << endl;


  if (argc < 3 || argc > 5)
    {
      cerr << "Usage: pgmcompare <file1> <file2> [<percent> [<type>]]" << endl
           << "       default for percent is " << percent << endl
           << "       types are: 0 (drawn) or 1 (scanned)" << endl;
      return 1;
    }
  if (argc > 3)
    {
      if (sscanf (argv[3], "%lf", &percent) != 1)
        {
          cerr << "Invalid percent: " << argv[3] << endl;
          return 1;
        }
    }
  
  if (argc > 4)
    {
      int i;
      if (sscanf (argv[4], "%d", &i) != 1 || (i & ~0x1))
        {
          cerr << "Invalid type: " << argv[4] << endl;
          return 1;
        }
      type = (imgtype)i;
    }
  
  try
    {
      img1->read (argv[1]);
    }
  catch (const exception &error)
    {
      cerr << "Reading first image: " << error.what () <<endl;
      return 1;
    }
  
  try
    {
      img2->read (argv[2]);
    }
  catch (const exception &error)
    {
      cerr << "Reading second image: " << error.what () <<endl;
      return 1;
    }

  sizecheck (img1);
  sizecheck (img2);

  /* Calculate the simple statistics */

  double mse;
  double rmse;
  double psnr;
  lq score;
  ImageComparison *comparison = new ImageComparison (*img1, *img2);

  mse  = comparison->cmpmse ();
  rmse = sqrt (mse); 
  psnr = (mse == 0? -1: comparison->cmpsnr ());
  
  cout.setf (ios::fixed, ios::floatfield);
  cout << "MSE: "  << std::setprecision (5) << std::setw (11) << mse << endl
       << "RMSE: " << std::setprecision (5) << std::setw (10) << rmse << endl
       << "PSNR: ";
  if (psnr < 0)
    {
      cout << "Infinite (identical images)" << endl;
    }
  else
    {
      cout << std::setprecision (5) << std::setw (10) << psnr << endl;
    }

  try 
    {
      score = comparison->distlq (percent, filter, type);
    }
  catch (exception &error)
    {
      cerr << "Performing comparison: " << error.what () << endl;
      cerr << "Not calculating L^q and Lq_d metrics." << endl;
      return 1;
    }

  double lqd_norm = score.details;

  delete comparison;

  /* Print the statistics */

  cout << "L^q: " << std::setprecision (5) << std::setw (11) 
       << score.colors + score.details << endl;
  cout << "Lq_d: " << std::setprecision (5) << std::setw (10) 
       << lqd_norm << endl;
  
  cout.setf (FMT0, ios::floatfield);

  /* Clean up */
  
  delete img1;
  delete img2;
  
  return 0;
}

