/*
 * Diff two images
 *
 * $Date: 2004/02/04 12:02:00 $
 * $Revision: 1.4 $
 *
 */

#include <iostream>
#include <stdexcept>
#include <stdio.h>
#include <WTools.hh>

#ifndef __WATCOMC__
using namespace std;
#endif

int 
main (int argc, char *argv[])
{
  cout << "pgmdiff version " << WAVE_VERSION << " (c) 2001-2003 by Herbert"
#ifdef PFI
       << " (pfi support enabled)"
#endif
       << endl;

  if (argc < 4 || argc > 5)
    {
      cerr << "Usage: pgmdiff <file1> <file2> <outfile> [magnify]" << endl;
      return 1;
    }
  Image *img1 = new StillImage ();
  Image *img2 = new StillImage ();
  coeff magnify = 1;

  if (argc > 4)
    {
      if (sscanf (argv[4], "%lf", &magnify) != 1)
        {
          cerr << "Invalid factor: " << argv[4] << endl;
          return 1;
        }
    }
  
  try 
    {
      img1->read (argv[1]);
    }
  catch (const exception &error)
    {
      cerr << "Error reading " << argv[1] << ": " << error.what () << endl;
      return 1;
    }
  
  try 
    {
      img2->read (argv[2]);
    }
  catch (const exception &error)
    {
      cerr << "Error reading " << argv[2] << ": " << error.what () << endl;
      return 1;
    }

  if (img1->size () != img2->size ())
    {
      cerr << "Error, image dimensions differ." << endl;
      return 1;
    }
  
  ImageComparison *comparison = new ImageComparison (*img1, *img2);
  Image *out = comparison->difference (magnify);
  delete comparison;
  
  try
    {
      out->write (argv[3], true);
    }
  catch (const exception &error)
    {
      cerr << "Error writing " << argv[3] << ": " << error.what () << endl;
      return 1;
    }
  
  return 0;
}
